#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 18-June-2003
#################################################################################
#
# Configure Ignore - Script part of Ethereal Realms, designed to manage 
#                    ignored user lists.
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Option;						# Option management
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $database;							# Database handle
my $cgi;							# Common gateway interface handle
my $login;							# Login handle
my $param;							# Parameter Handle

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Retreive hash
	$database->GetHashConfiguration(\%config);
	

	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new eRealms::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Screen display
		InitialDisplay($database, $cgi, $param, \%config);
	}

	# Document footer
	$database->DocumentGetFooter($database);

#################################################################################
# Sub-Routines
#################################################################################

#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash

	my $option;						# Option handler
	my $table;						# Table handle
	my $tmpl;						# Template handle

	my $res;						# Results
	my $statement;						# Statement

	my $puppet;						# Choice of the matter	
	my $user;						# Username

	my $confirm;						# Confirmation fund
	my $ignored;						# List of the dammed
	my $types;						# Available ignore types

	my @types;						# Type of ignores
	my @ignored;						# List of the dammed
	

	#####################
	# Program area	
	
	# Create instances
	$option   = new eRealms::Option();
	$table    = new eRealms::Table($cgi);
	$tmpl     = new eRealms::Template($config);


	# Information handling
	# Addition into list
	if (defined($param->{$config->{'TxtSubmitAddition'}}))
	{
		# Determine existence
		($puppet, $login) = $database->DataGet("SELECT
			  PuppetName,
			  PuppeteerLogin
			 FROM Puppet
			 WHERE LOWER(PuppetName)=?", 

		 lc($param->{'IGNORE'})
		);


		# Discover
		$types = $option->FindIgnore($config->{'OptIgnore'}, $param->{'TYPE'});


		# Verification
		if (defined($puppet))
		{
			# Global or specific
			if ($types eq 'global')
			{
				# Block both puppet and puppeteer
				# Insert information
				$database->Quick("INSERT INTO PuppetIgnore 
					(PuppeteerLogin,
					 PuppetIgnore,
					 PuppetName,
					 PuppetLogin)
					VALUES (?,?,?,?)", 

			 	 $param->{'USER'}, 
				 $types,
				 $puppet,
				 $login
				);
			}
			else
			{
				# Simply a puppet
				# Insert information
				$database->Quick("INSERT INTO PuppetIgnore 
					(PuppeteerLogin,
					 PuppetIgnore,
					 PuppetName)
					VALUES (?,?,?)",

			 	 $param->{'USER'}, 
				 $types,
				 $puppet
				);
			}
		}
	}

	# Removal from list
	elsif (defined($param->{$config->{'TxtSubmitDelete'}}))
	{
		# Only if confirmed
		if (defined($param->{'CONFIRM'}))
		{
			# Remove descriptor
			$param->{'REMOVE'} =~ s/^(\+|\-|\*|\~)\s(.+)/$2/;

			# Remove ignored person
			$database->Quick("DELETE FROM PuppetIgnore 
				WHERE PuppeteerLogin=? 
				  AND PuppetName=?", 

			 $param->{'USER'}, 
			 $param->{'REMOVE'}
			);
		}
	}

	# Cleanup
	$param->Cleanup('CONFIRM', 'GLOBAL');


	# Types
	$option->Split(\@types, $config->{'OptIgnore'});


	# Pretty adjustment
	$database->Pull(\$statement, "SELECT
		 PuppetName   AS \"PuppetName\",
		 PuppetIgnore AS \"PuppetIgnore\"
		FROM PuppetIgnore
		WHERE PuppeteerLogin=?
		ORDER BY PuppetName",

	 $param->{'USER'}
	);

	# Loop and create
	while ($res = $statement->fetchrow_hashref())
	{
		# Indicator
		# One
		if ($res->{'PuppetIgnore'} eq 'single')
		{
			# Normal application
			$puppet = "- $res->{PuppetName}";
		}

		# Many
		elsif ($res->{'PuppetIgnore'} eq 'global')
		{
			# Anti-social behaviour
			$puppet = "+ $res->{PuppetName}";
		}

		# Photographic
		elsif ($res->{'PuppetIgnore'} eq 'visual')
		{
			# Virgin eyes syndrome
			$puppet = "* $res->{PuppetName}";
		}

		# Highlight
		else
		{
			# Virgin eyes syndrome
			$puppet = "~ $res->{PuppetName}";
		}

		# Push onto array
		push(@ignored, $puppet);

	}
	$statement->finish();

	
	# Generate widgets
	$confirm = $cgi->submit($config->{'TxtSubmitDelete'}) . '&nbsp' . $cgi->checkbox(-name=>'CONFIRM', -label=>'');
	$ignored = $cgi->scrolling_list('REMOVE', \@ignored, $ignored[0], 10);
	$types   = $cgi->radio_group('TYPE', \@types, $types[0], 'true');


	# Title
	$tmpl->ShowMain($config->{'TagIgnoreTitle'});

	# Ignore list
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Display of information
	$table->MakeTop();

	# Help
	$table->MakeSingle($config->{'HlpIgnore'});


	# Additions
	$table->MakeSingle($tmpl->PassTitle($config->{'TagIgnoreAdd'}));
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagIgnorePuppet'}),
	 $cgi->textfield('IGNORE', '', 20, 30));
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagIgnoreType'}),
	 $types);

	$table->MakeBlank();
	$table->MakeValid('&nbsp;', $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBlank();
	$table->MakeBlank();

	# Removal
	$table->MakeSingle($tmpl->PassTitle($config->{'TagIgnoreRem'}));
	$table->MakeValid($ignored, $confirm, 'middle');
	$table->MakeBottom();

	# From bottom
	print $cgi->end_form(), "\n";
}
